@@ -599,6 +599,14 @@ def removeDir(rmDir):
     rmtree(rmDir)
     return True
 
+def clearDirectory(dn):
+    """
+    Очистить содержимое директории
+    :param dn:
+    :return:
+    """
+    for dn in listDirectory(dn, fullPath=True):
+        removeDir(dn)
 
 def check_rw(dn):
     """
@@ -787,17 +795,22 @@ def listDirectory(directory, fullPath=False, onlyDir=False):
     return []
 
 
-def makeDirectory(pathname):
-    """Make directory and parent.
+def makeDirectory(pathname, force=False):
+    """
+    Создать каталог вместе с вышестоящими. force - используется для удаления
+    файла, который является частью пути.
 
-    Return False on error"""
+    Возвращает False при ошибке"""
     try:
         parent = path.split(path.normpath(pathname))[0]
         if not path.exists(parent):
             makeDirectory(parent)
         else:
             if path.exists(pathname):
-                return True
+                if force and not path.isdir(pathname):
+                    os.unlink(pathname)
+                else:
+                    return True
         os.mkdir(pathname)
         return True
     except (OSError, IOError):
@@ -1651,3 +1664,108 @@ def sha256sum(*fns):
                 b = f.read(512 * kb)
                 hasher.update(b)
     return hasher.hexdigest()
+
+def forceFileRename(source_fn, target_fn):
+    """
+    Принудительно перенести файл, даже если назначение занято
+    :param source_fn:
+    :param target_fn:
+    :return:
+    """
+    if path.exists(target_fn) and not path.isfile(target_fn):
+        if path.isdir(target_fn):
+            removeDir(target_fn)
+        else:
+            os.unlink(target_fn)
+    os.rename(source_fn, target_fn)
+
+
+def move_files(source, target):
+    """
+    Переменстить все файлы из source в target вместе со структурой
+    :param source:
+    :param target:
+    :return:
+    """
+    for fn in find(source, filetype=FindFileType.RegularFile,
+                   fullpath=False):
+        source_fn = path.join(source, fn)
+        target_fn = path.join(target, fn)
+        target_dn = path.dirname(target_fn)
+        # принудительно создать каталог, удалив файлы являющиейся частью пути
+        makeDirectory(target_dn, force=True)
+        forceFileRename(source_fn, target_fn)
+    clearDirectory(source)
+
+class RCSError(Exception):
+    pass
+
+class DirectoryRCS(object):
+    """
+    Объект оперирует тремя каталогами, в котором хранит:
+    фиксированную версию данных
+    изменения для фиксирования данных (без учёта удаления файлов)
+    рабочая версия данных
+    """
+    def __init__(self, workdir, indexdir, fixeddir):
+        self.workdir = workdir
+        self.indexdir = indexdir
+        self.fixeddir = fixeddir
+
+    def is_clear(self):
+        """
+        Изменения есть только в workdir или в fixed
+        :return:
+        """
+        return (isEmpty(self.indexdir) and isEmpty(self.workdir) and
+                not isEmpty(self.fixeddir))
+
+    def is_worked(self):
+        """
+        Есть непроиндексированные изменения
+        :return:
+        """
+        return not isEmpty(self.workdir)
+
+    def not_prepared(self):
+        return (isEmpty(self.indexdir) and not isEmpty(self.workdir) and
+                isEmpty(self.fixeddir))
+
+    def fixing(self):
+        """
+        Зафиксировать все изменения
+        :return:
+        """
+        if not isEmpty(self.indexdir):
+            self.movedata(self.indexdir, self.fixeddir)
+
+        if path.exists(self.indexdir):
+            removeDir(self.indexdir)
+        if not isEmpty(self.workdir):
+            self.movedata(self.workdir, self.fixeddir)
+
+    def indexing(self):
+        """
+        Зафиксировать рабочие изменения в индекс
+        :return:
+        """
+        self.movedata(self.workdir, self.indexdir)
+
+    def movedata(self, source, target):
+        """
+        Поместить все work изменения в index
+        :return:
+        """
+        if not path.exists(target) and path.exists(source):
+            try:
+                os.rename(source, target)
+            except OSError as e:
+                raise RCSError(_("Failed to rename data: %s") % str(e))
+            makeDirectory(source)
+        else:
+            try:
+                move_files(source, target)
+            except FilesError as e:
+                raise RCSError(_("Failed to move data: %s") % str(e))
+
+
